@@ -7,10 +7,11 @@ from logging import INFO as INFO_LOGGING
 from logging import WARNING as WARNING_LOGGING
 from logging import Logger, getLogger
 from types import TracebackType
-from typing import Any, Protocol, Self, final, runtime_checkable
+from typing import Any, ClassVar, Literal, Protocol, Self, runtime_checkable
 from uuid import UUID, uuid4
 
 from haiway.context.identifier import ScopeIdentifier
+from haiway.state import Immutable
 from haiway.types import Missing
 from haiway.utils.formatting import format_str
 
@@ -22,6 +23,7 @@ __all__ = (
     "ObservabilityEventRecording",
     "ObservabilityLevel",
     "ObservabilityLogRecording",
+    "ObservabilityMetricKind",
     "ObservabilityMetricRecording",
     "ObservabilityScopeEntering",
     "ObservabilityScopeExiting",
@@ -117,6 +119,9 @@ class ObservabilityEventRecording(Protocol):
     ) -> None: ...
 
 
+type ObservabilityMetricKind = Literal["counter", "histogram", "gauge"]
+
+
 @runtime_checkable
 class ObservabilityMetricRecording(Protocol):
     """
@@ -135,6 +140,7 @@ class ObservabilityMetricRecording(Protocol):
         metric: str,
         value: float | int,
         unit: str | None,
+        kind: ObservabilityMetricKind,
         attributes: Mapping[str, ObservabilityAttribute],
     ) -> None: ...
 
@@ -190,7 +196,7 @@ class ObservabilityScopeExiting(Protocol):
     ) -> None: ...
 
 
-class Observability:  # avoiding State inheritance to prevent propagation as scope state
+class Observability(Immutable):  # avoiding State inheritance to prevent propagation as scope state
     """
     Container for observability recording functions.
 
@@ -201,107 +207,13 @@ class Observability:  # avoiding State inheritance to prevent propagation as sco
     This class is immutable after initialization.
     """
 
-    __slots__ = (
-        "attributes_recording",
-        "event_recording",
-        "log_recording",
-        "metric_recording",
-        "scope_entering",
-        "scope_exiting",
-        "trace_identifying",
-    )
-
-    def __init__(
-        self,
-        trace_identifying: ObservabilityTraceIdentifying,
-        log_recording: ObservabilityLogRecording,
-        metric_recording: ObservabilityMetricRecording,
-        event_recording: ObservabilityEventRecording,
-        attributes_recording: ObservabilityAttributesRecording,
-        scope_entering: ObservabilityScopeEntering,
-        scope_exiting: ObservabilityScopeExiting,
-    ) -> None:
-        """
-        Initialize an Observability container with recording functions.
-
-        Parameters
-        ----------
-        trace_identifying: ObservabilityTraceIdentifying
-            Function for identifying traces
-        log_recording: ObservabilityLogRecording
-            Function for recording log messages
-        metric_recording: ObservabilityMetricRecording
-            Function for recording metrics
-        event_recording: ObservabilityEventRecording
-            Function for recording events
-        attributes_recording: ObservabilityAttributesRecording
-            Function for recording attributes
-        scope_entering: ObservabilityScopeEntering
-            Function called when a scope is entered
-        scope_exiting: ObservabilityScopeExiting
-            Function called when a scope is exited
-        """
-        self.trace_identifying: ObservabilityTraceIdentifying
-        object.__setattr__(
-            self,
-            "trace_identifying",
-            trace_identifying,
-        )
-        self.log_recording: ObservabilityLogRecording
-        object.__setattr__(
-            self,
-            "log_recording",
-            log_recording,
-        )
-        self.metric_recording: ObservabilityMetricRecording
-        object.__setattr__(
-            self,
-            "metric_recording",
-            metric_recording,
-        )
-        self.event_recording: ObservabilityEventRecording
-        object.__setattr__(
-            self,
-            "event_recording",
-            event_recording,
-        )
-        self.attributes_recording: ObservabilityAttributesRecording
-        object.__setattr__(
-            self,
-            "attributes_recording",
-            attributes_recording,
-        )
-        self.scope_entering: ObservabilityScopeEntering
-        object.__setattr__(
-            self,
-            "scope_entering",
-            scope_entering,
-        )
-        self.scope_exiting: ObservabilityScopeExiting
-        object.__setattr__(
-            self,
-            "scope_exiting",
-            scope_exiting,
-        )
-
-    def __setattr__(
-        self,
-        name: str,
-        value: Any,
-    ) -> Any:
-        raise AttributeError(
-            f"Can't modify immutable {self.__class__.__qualname__},"
-            f" attribute - '{name}' cannot be modified"
-        )
-
-    def __delattr__(
-        self,
-        name: str,
-    ) -> None:
-        raise AttributeError(
-            f"Can't modify immutable {self.__class__.__qualname__},"
-            f" attribute - '{name}' cannot be deleted"
-        )
+    trace_identifying: ObservabilityTraceIdentifying
+    log_recording: ObservabilityLogRecording
+    metric_recording: ObservabilityMetricRecording
+    event_recording: ObservabilityEventRecording
+    attributes_recording: ObservabilityAttributesRecording
+    scope_entering: ObservabilityScopeEntering
+    scope_exiting: ObservabilityScopeExiting
 
 
 def _logger_observability(
@@ -371,6 +283,7 @@ def _logger_observability(
         metric: str,
         value: float | int,
         unit: str | None,
+        kind: ObservabilityMetricKind,
         attributes: Mapping[str, ObservabilityAttribute],
     ) -> None:
         if attributes:
@@ -408,7 +321,7 @@ def _logger_observability(
     ) -> None:
         logger.log(
             ObservabilityLevel.DEBUG,
-            f"[{trace_id_hex}] {scope.unique_name} Entering scope: {scope.label}",
+            f"[{trace_id_hex}] {scope.unique_name} Entering scope: {scope.name}",
         )
 
     def scope_exiting(
@@ -419,7 +332,7 @@ def _logger_observability(
     ) -> None:
         logger.log(
             ObservabilityLevel.DEBUG,
-            f"{scope.unique_name} Exiting scope: {scope.label}",
+            f"{scope.unique_name} Exiting scope: {scope.name}",
             exc_info=exception,
         )
 
@@ -434,8 +347,7 @@ def _logger_observability(
     )
 
 
-@final
-class ObservabilityContext:
+class ObservabilityContext(Immutable):
     """
     Context manager for observability within a scope.
 
@@ -446,7 +358,7 @@ class ObservabilityContext:
     This class is immutable after initialization.
     """
 
-    _context = ContextVar[Self]("ObservabilityContext")
+    _context: ClassVar[ContextVar[Self]] = ContextVar[Self]("ObservabilityContext")
 
     @classmethod
     def scope(
@@ -486,7 +398,7 @@ class ObservabilityContext:
                     resolved_observability = observability
 
                 case None:
-                    resolved_observability = _logger_observability(getLogger(scope.label))
+                    resolved_observability = _logger_observability(getLogger(scope.name))
 
                 case Logger() as logger:
                     resolved_observability = _logger_observability(logger)
@@ -534,7 +446,7 @@ class ObservabilityContext:
         Returns
         -------
         str
-            The hexadecimal representation of the trace ID
+            The string representation of the trace ID
 
         Raises
         ------
@@ -542,12 +454,10 @@ class ObservabilityContext:
             If called outside of any scope context
         """
         try:
-            return (
-                cls._context.get()
-                .observability.trace_identifying(
+            return str(
+                cls._context.get().observability.trace_identifying(
                     scope_identifier if scope_identifier is not None else ScopeIdentifier.current()
                 )
-                .hex
             )
 
         except LookupError as exc:
@@ -649,6 +559,7 @@ class ObservabilityContext:
         *,
         value: float | int,
         unit: str | None,
+        kind: ObservabilityMetricKind,
         attributes: Mapping[str, ObservabilityAttribute],
     ) -> None:
         """
@@ -667,6 +578,8 @@ class ObservabilityContext:
             The numeric value of the metric
         unit: str | None
             Optional unit for the metric (e.g., "ms", "bytes")
+        kind: ObservabilityMetricKind
+            The metric kind defining its value handling.
         attributes: Mapping[str, ObservabilityAttribute]
             Key-value attributes associated with the metric
         """
@@ -680,6 +593,7 @@ class ObservabilityContext:
                     metric=metric,
                     value=value,
                     unit=unit,
+                    kind=kind,
                     attributes=attributes,
                 )
 
@@ -728,55 +642,31 @@ class ObservabilityContext:
                 exception=exc,
             )
 
-    __slots__ = (
-        "_scope",
-        "_token",
-        "observability",
-    )
+    _scope: ScopeIdentifier
+    observability: Observability
+    _token: Token[Self] | None = None
 
     def __init__(
         self,
         scope: ScopeIdentifier,
         observability: Observability | None,
     ) -> None:
-        self._scope: ScopeIdentifier
         object.__setattr__(
             self,
             "_scope",
             scope,
         )
-        self.observability: Observability
         object.__setattr__(
             self,
             "observability",
             observability,
         )
-        self._token: Token[ObservabilityContext] | None
         object.__setattr__(
             self,
             "_token",
             None,
         )
 
-    def __setattr__(
-        self,
-        name: str,
-        value: Any,
-    ) -> Any:
-        raise AttributeError(
-            f"Can't modify immutable {self.__class__.__qualname__},"
-            f" attribute - '{name}' cannot be modified"
-        )
-
-    def __delattr__(
-        self,
-        name: str,
-    ) -> None:
-        raise AttributeError(
-            f"Can't modify immutable {self.__class__.__qualname__},"
-            f" attribute - '{name}' cannot be deleted"
-        )
-
     def __enter__(self) -> None:
         """
         Enter this observability context.
@@ -822,7 +712,7 @@ class ObservabilityContext:
             If the context is not active
         """
         assert self._token is not None, "Unbalanced context enter/exit"  # nosec: B101
-        ObservabilityContext._context.reset(self._token)
+        ObservabilityContext._context.reset(self._token)  # pyright: ignore[reportArgumentType]
         object.__setattr__(
             self,
             "_token",
